#!/usr/bin/perl

# Thanks to http://www.gigi.co.uk/guiconsole/serviceexample.txt for the Example Script!

use warnings;
no warnings 'uninitialized', 'once';

use strict;
use Win32;
use Win32::Daemon;
use Win32::EventLog::Message;
use Getopt::Long;

#--------------------------------------------------------------
# service setup
#--------------------------------------------------------------

Win32::Daemon::RegisterCallbacks( {
    start       =>  \&__callback_start,
    running     =>  \&__callback_running,
    stop        =>  \&__callback_stop,
    pause       =>  \&__callback_pause,
    continue    =>  \&__callback_continue,
} );

my $cmdpath = join( '', Win32::GetFullPathName( $0 ) );

# name          The 'internal' service name; that is, the name of the registry key used to store the information on this service.
# display       The 'display' service name; that is, the name displayed by the services control panel or MMC plugin.
# path          The full path name to the executable. This should be the path to your Perl executable, which will normally be the contents of $^X.
# user          The username the service is to run under; this is optional.
# password      The password to be used to log in the service; this is technically optional, but needs to be specified if {user} is.
# parameters    The parameters to be passed to Perl; in other words, the command line you would execute interactively, but without the leading ``perl ''.
# machine       The name of the machine to create the service on. Omission or an empty string specify the machine executing the call.
# service_type  An integer representing the type of the service; defaults to SERVICE_WIN32_OWN_PROCESS.
# start_type    An integer specifying how (or whether) the service is to be started. The default is SERVICE_AUTO_START.
# error_control An integer specifying how the Service Control Manager is to react if the service fails to start. The default is SERVICE_ERROR_IGNORE, which in fact gets you an error log entry.
# load_order    The name of the load order group of which this service is a member. The default is membership in no group. See value ServiceGroupOrder in registry key HKEY_LOCAL_MACHINE\System\CurrentControlSet\Control for the names.
# tag_id        An integer representing the startup order of the service within its load ordering group.
# dependencies  A reference to the 'internal' names of services and/or load ordering groups upon which this service depends. The default is no dependencies. Load order group names are prefixed with a '+' to distinguish them from service names.
# description   A short text description of the service, displayed (at least) as flyover help by the MMC ``services'' plugin.
# logmethod     'eventlog' = to event log (default)
#               'stdout'   = to stdout 

our $serviceconfig = { name => 'raptorfast',
                       display => 'PCCC Raptor FAST',
                       path => $^X,      # this is correct if you are running as "perl script.pl"
                       parameters => $cmdpath,
                       user    =>  '',
                       pwd     =>  '',
                       machine =>  '',
                       description => 'PCCC Raptor Forward And Store Technology',
                       logmethod => 'eventlog', 
                       service_type => SERVICE_WIN32_OWN_PROCESS | SERVICE_INTERACTIVE_PROCESS,
                     };

my %cmdopts = ();

my $result = GetOptions( \%cmdopts,
                      qw(
                        install|i
                        remove|r
                        
                       ) );

if( $cmdopts{install} ) {
    &Install();
    exit();
} elsif( $cmdopts{remove} ) {
    Remove();
    exit();
}

my %initcontext = (
last_state => SERVICE_RUNNING,
count   =>  0,
start_time => time(),
);

# start with 2 second wait on continue loop
my $service = Win32::Daemon::StartService( \%initcontext, 2000 );

print "DEBUG: here";

EventLog("Info: " . GetError());

sub GetError {
    return( Win32::FormatMessage( Win32::Daemon::GetLastError() ) );
}

#--------------------------------------------------------------
# service Install
#--------------------------------------------------------------

sub Install {
    Win32::EventLog::Message::RegisterSource( 'Application', $serviceconfig->{display} );
    if( Win32::Daemon::CreateService( $serviceconfig ) )  {
        print "The $serviceconfig->{display} was successfully installed.\n";
    } else {
        print "Failed to add the $serviceconfig->{display} service.\nError: " . GetError() . "\n";
    }
}

#--------------------------------------------------------------
# service Remove
#--------------------------------------------------------------

sub Remove {
    if( Win32::Daemon::DeleteService( $serviceconfig->{name} ) ) {
        print "The $serviceconfig->{display} was successfully removed.\n";
    } else {
        print "Failed to remove the $serviceconfig->{display} service.\nError: " . GetError() . "\n";
    }
}

#--------------------------------------------------------------
# ServiceStart
#--------------------------------------------------------------

sub ServiceStart {
    EventLog("Info: $serviceconfig->{display} starting.");
    
}

#--------------------------------------------------------------
# ServicePause
#--------------------------------------------------------------

sub ServicePause {
    EventLog("Info: $serviceconfig->{display} pausing.");
}

#--------------------------------------------------------------
# ServiceContinue
#--------------------------------------------------------------

sub ServiceContinue {
    EventLog("Info: $serviceconfig->{display} continuing.");
    }

#--------------------------------------------------------------
# ServiceRunning
#--------------------------------------------------------------

sub ServiceRunning {
    
    print "DEBUG: stuff # do prog stuff here..........\n";
    
    EventLog("Info: $serviceconfig->{display} running.");
    
}

#--------------------------------------------------------------
# ServiceStop
#--------------------------------------------------------------

sub ServiceStop {
    EventLog("Info: $serviceconfig->{display} stopping.");
}

#--------------------------------------------------------------
# EventLog
#--------------------------------------------------------------

sub EventLog {
    my($msg) = @_;

    print "$msg\n";
    if($serviceconfig->{logmethod} eq 'stdout') {
        # stdout logging requested
        print $msg, "\n";
        return;
    }
    return unless $msg =~ /^\S+:/;
    
    # ignore debug messages for event log
    return if $msg =~ /^debug:/i;
    
    my $EventType = Win32::EventLog::EVENTLOG_INFORMATION_TYPE();
    $EventType = Win32::EventLog::EVENTLOG_WARNING_TYPE() if $msg =~ /^w\S+:/i;
    $EventType = Win32::EventLog::EVENTLOG_ERROR_TYPE()   if $msg =~ /^e\S+:/i;
    $msg =~ s/^\S+:\s*//;
    
    my $strings = "$msg\n\n  http://www.pccc.com/\n";
    
    my $EventLog = Win32::EventLog->new( 'Application' );
   
   $EventLog->Report({
       Source   =>  $serviceconfig->{display},
       EventType => $EventType,
       Category  => 0,
       EventID   => 0,
       Strings   => $strings,
       Data      => "",
   });
  
}

###############################################################
# 
# internal service callback handling
# no user servicable parts below :-)
#--------------------------------------------------------------

sub __callback_running
{
    my( $Event, $Context ) = @_;
    
    # Note that here you want to check that the state
    # is indeed SERVICE_RUNNING. Even though the Running
    # callback is called it could have done so before 
    # calling the "Start" callback.
    if( SERVICE_RUNNING == Win32::Daemon::State() )
    {
        ServiceRunning();
        
    }
    
}    

sub __callback_start
{
    my( $Event, $Context ) = @_;
    # Initialization code
    # ...do whatever you need to do to start...
    $Context->{last_state} = SERVICE_RUNNING;
    $Context->{count}++;
    ServiceStart();
    Win32::Daemon::State( &SERVICE_RUNNING );
}

sub __callback_pause
{
    my( $Event, $Context ) = @_;
    $Context->{last_state} = SERVICE_PAUSED;
    ServicePause();
    Win32::Daemon::State( SERVICE_PAUSED );
}

sub __callback_continue
{
    my( $Event, $Context ) = @_;
    $Context->{last_state} = SERVICE_RUNNING;
    ServiceContinue();
    Win32::Daemon::State( SERVICE_RUNNING );
}

sub __callback_stop
{
    my( $Event, $Context ) = @_;
    $Context->{last_state} = SERVICE_STOPPED;
    ServiceStop();
    Win32::Daemon::State( SERVICE_STOPPED );
    # We need to notify the Daemon that we want to stop callbacks and the service.
    Win32::Daemon::StopService();
}

